#!/bin/bash
#

usage()
{
   echo "USAGE: `basename ${0}` <product_dir> [<basequal>] [tar]"
}

# -------------------------------------------------------------------
# shared boilerplate
# -------------------------------------------------------------------

get_this_dir() 
{
    ( cd / ; /bin/pwd -P ) >/dev/null 2>&1
    if (( $? == 0 )); then
      pwd_P_arg="-P"
    fi
    reldir=`dirname ${0}`
    thisdir=`cd ${reldir} && /bin/pwd ${pwd_P_arg}`
}

get_ssibuildshims()
{
    # make sure we can use the setup alias
    if [ -z ${UPS_DIR} ]
    then
       echo "ERROR: please setup ups"
       exit 1
    fi
    source `${UPS_DIR}/bin/ups setup ${SETUP_UPS}`
    
    setup ssibuildshims ${ssibuildshims_version} -z ${product_dir}
}

set_qualifiers()
{
  if [[ -n "${basequal}" ]]; then
    qlist=(`echo ${basequal} | sed -e 's/:/ /g' | tr " " "\n"| sort |tr "\n" " "`)
    use_apple_clang=false
    for qual in "${qlist[@]}"; do
      case ${qual} in
        e14)
          cc=gcc
          cxx=g++
          cxxflg="-std=c++14 -Wno-deprecated-declarations"
          ;;
        *)
          ssi_die "Unrecognized qualifier ${qual}"
      esac
    done
  else # No qualifier.
    if [ ${OS1} = "Darwin" ]; then
      use_apple_clang=true
    else # Linux
      local OIFS=$IFS; IFS=.; local gccv=($(gcc -dumpversion)); IFS="$OIFS"
      if (( ${gccv[0]} < 4 )) || \
        { (( ${gccv[0]} == 4 )) && (( ${gccv[1]} < 8 )); }; then
        ssi_die "Unqualified build requires GCC >= 4.8.0"
      fi
    fi
    cc=cc
    cxx=c++
  fi
}

unpack_and_patch()
{
    mkdir -p ${sourcedir}

    # unpack
    llvmtar=${tardir}/clang-${pkgdotver}.tar.bz2
    if [[ -d "${sourcedir}/llvm-${pkgdotver}" ]]; then
      echo "clang source already exists."
    fi
    cd ${sourcedir}
    [[ -d llvm ]] || ( tar -xf ${tardir}/llvm-${pkgdotver}.src.tar.xz && mv llvm-${pkgdotver}.src llvm )
    [[ -d llvm/projects/libcxx ]] || ( tar -xf ${tardir}/libcxx-${pkgdotver}.src.tar.xz && mv libcxx-${pkgdotver}.src llvm/projects/libcxx )
    [[ -d llvm/projects/libcxxabi ]] || ( tar -xf ${tardir}/libcxxabi-${pkgdotver}.src.tar.xz && mv libcxxabi-${pkgdotver}.src llvm/projects/libcxxabi )
    [[ -d llvm/tools/clang ]] || ( tar -xf ${tardir}/cfe-${pkgdotver}.src.tar.xz && mv cfe-${pkgdotver}.src llvm/tools/clang )
    [[ -d llvm/tools/clang/tools/extra ]] || ( tar -xf ${tardir}/clang-tools-extra-${pkgdotver}.src.tar.xz && mv clang-tools-extra-${pkgdotver}.src llvm/tools/clang/tools/extra )
    cd llvm/tools/clang
    patchdir=${product_dir}/${package}/${pkgver}/patch
    for patch in ${patchdir}/clang-env-var-gcc-toolchain.patch; do
      patch ${DEBUG_PATCHES:+--verbose} -p1 < "${patch}" || ssi_die "application of patch ${patch} failed."
    done
    (( ${DEBUG_PATCHES:+1} )) && exit 1
}

# -------------------------------------------------------------------
# start processing
# -------------------------------------------------------------------

product_dir=${1}
basequal=${2}
if [[ "${basequal}" == tar ]]; then
  maketar=${basequal}
  unset basequal
else
  maketar=${3}
fi

if [ -z ${product_dir} ]
then
   echo "ERROR: please specify the local product directory"
   usage
   exit 1
fi

# -------------------------------------------------------------------
# package name and version
# -------------------------------------------------------------------

package=clang
origpkgver=v4_0_1
origpkgdotver=`echo ${origpkgver} | sed -e 's/_/./g' | sed -e 's/^v//'`
pkgver=${origpkgver//-/_}
pkgdotver=`echo ${pkgver} | sed -e 's/_/./g' | sed -e 's/^v//'`
ssibuildshims_version=v0_19_00
python_version=v2_7_13d

get_this_dir

get_ssibuildshims

if [[ -n "${basequal}" ]]; then
  bqspec=NO_TYPE
else
  bqspec=NO_QUAL
fi

source define_basics ${bqspec}

set_qualifiers

echo "building ${package} ${pkgver} ${fullqual:+-q ${fullqual}} for ${OS}-${plat} (flavor ${flvr})"

if [ ${OS1} = "Darwin" ]
then
  prefix="${pkgdir}/Toolchains/LLVM${origpkgdotver}.xctoolchain/usr"
  echo "DYLD_LIBRARY_PATH $DYLD_LIBRARY_PATH"
  export DYLD_LIBRARY_PATH=${prefix}/lib64:${prefix}/lib
  shext=dylib
  extra_opts=(-DLLVM_CREATE_XCODE_TOOLCHAIN:BOOL=ON \
    -DLLVM_ENABLE_LTO:STRING=Full)
  # Install in a way which is compatible with acting as a toolchain for
  # XCode.
  install_target=install-xcode-toolchain
else
  prefix="${pkgdir}"
  echo "LD_LIBRARY_PATH $LD_LIBRARY_PATH"
  export LD_LIBRARY_PATH=${prefix}/lib64:${prefix}/lib
  shext=so
  install_target=install
  if [[ "$OS" == "slf6"* ]]; then
    setup -B -z ${product_dir} -z ${PRODUCTS} python ${python_version} || \
      ssi_die "Unable to set up required python ${python_version} on platform $OS."
  fi
fi

if [ "${maketar}" = "tar" ] && [ -d ${prefix}/lib ]
then
  # N.B. explicitly exclude source
  ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball \
    -d data:doc:include:scripts:example:share \
    ${product_dir} \
    ${package} \
    ${pkgver} \
    ${fullqual:-NO_QUAL}
   exit $?
fi

# -------------------------------------------------------------------
# package dependent stuff goes here
# -------------------------------------------------------------------

mkdir -p ${pkgdir}
if [ ! -d ${pkgdir} ]
then
   echo "ERROR: failed to create ${pkgdir}"
   exit 1
fi
mkdir -p ${blddir}
if [ ! -d ${blddir} ]
then
   echo "ERROR: failed to create ${blddir}"
   exit 1
fi

[[ -d "${sourcedir}" ]] || unpack_and_patch

source setup_cmake

num_cores=$(${SSIBUILDSHIMS_DIR}/bin/ncores)

set -x

# declare now so we can setup
# fake ups declare
fakedb=${product_dir}/${package}/${pkgver}/fakedb
${SSIBUILDSHIMS_DIR}/bin/fake_declare_product ${product_dir} ${package} ${pkgver} ${fullqual:-NO_QUAL}

setup -B ${package} ${pkgver} ${fullqual:+-q ${fullqual}} -z ${fakedb}:${PRODUCTS} || ssi_die "ERROR: fake setup failed"

# remove /usr/local/bin and . from path
PATH=`dropit -e /usr/local/bin .`
# remove /usr/local/X from LD_LIBRARY PATH and DYLD_LIBRARY_PATH
LD_LIBRARY_PATH=${LD_LIBRARY_PATH:+$(dropit -a -p "$LD_LIBRARY_PATH" /usr/local/)}
DYLD_LIBRARY_PATH=${DYLD_LIBRARY_PATH:+$(dropit -a -p "$DYLD_LIBRARY_PATH" /usr/local/)}
# make sure libraries are in the path
export PATH=${pkgdir}/bin:${PATH}
export COMPILER_PATH=${pkgdir}/bin
# need proper environment
echo ""
echo "CC $cc CXX $cxx "
echo "LD_LIBRARY_PATH $LD_LIBRARY_PATH"
echo "DYLD_LIBRARY_PATH $DYLD_LIBRARY_PATH"
echo "COMPILER_PATH   $COMPILER_PATH"

cd ${blddir}
CC=${cc} CXX=${cxx} \
  cmake ../../source/llvm -DCMAKE_INSTALL_PREFIX=${pkgdir} \
  -DCMAKE_BUILD_TYPE=Release \
  -DLLVM_BUILD_LLVM_DYLIB:BOOL=ON \
  -DLLVM_TARGETS_TO_BUILD="X86" \
  -DLLVM_PARALLEL_COMPILE_JOBS:STRING=${num_cores} \
  -DLLVM_PARALLEL_LINK_JOBS:STRING=${num_cores} \
  "${extra_opts[@]}" \
  -DLLVM_BINUTILS_INCDIR=$(${cc} -print-file-name=plugin)/include || \
  ssi_die "Unable to configure clang."

make -j "${num_cores}" || ssi_die "Unable to make clang."
make -j "${num_cores}" ${install_target} || ssi_die "Unable to install clang."

set +x

if [ ! -d ${prefix}/bin ]
then
   echo "ERROR: failed to create ${prefix}/bin"
   echo "       Something is very wrong"
   exit 1
fi

if [ ! -d ${prefix}/lib ]
then
   echo "ERROR: failed to create ${prefix}/lib"
   echo "       Something is very wrong"
   exit 1
fi

# real ups declare
"${SSIBUILDSHIMS_DIR}/bin/declare_product" "${product_dir}" ${package} ${pkgver} ${fullqual:-NO_QUAL}

# -------------------------------------------------------------------
# common bottom stuff
# -------------------------------------------------------------------

# this should not complain
echo "Finished building ${package} ${pkgver} ${fullqual:+-q ${fullqual}}"
source `ups setup -z "${product_dir}" -z "${PRODUCTS}" ${package} ${pkgver} ${fullqual:+-q ${fullqual}}`
echo "${package} ${pkgver} ${fullqual:+-q ${fullqual}} is installed at ${CLANG_FQ_DIR}"

if [ "${maketar}" = "tar" ]; then
  # N.B. explicitly exclude source
  ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball \
    -d data:doc:include:scripts:example:share \
    ${product_dir} \
    ${package} \
    ${pkgver} \
    ${fullqual:-NO_QUAL}
fi

exit 0
